
{$i-}            { disable I/O checking     }
{$r-}            { disable range checking   }
{$v-}            { disable string checking  }
{=========================================================================}
{                                                                         }
{           lowlib           a collection of routines for low-level       }
{                              memory and data manipulation               }
{           version          1.0                                          }
{           date             13 Aug 1984                                  }
{                                                                         }
{=========================================================================}

unit LowLib;
intrinsic code 22;
interface

type

  Byte                       = 0..255;
  Buffer                     = packed array[0..511] of Byte;
  HeapPtr =
    packed record
      case Byte of
        0  :  (Addr  : Integer);    { address of various pointers       }
        1  :  (Ptr   : ^Integer);   { used for mark/release of heap     }
        2  :  (Str   : ^String);    { used for string manipulation      }
        3  :  (Buf   : ^Buffer)     { used for raw data manipulation    }
    end;

{=========================================================================}
{                         procedures and functions                        }
{=========================================================================}
{
  Shl           returns Val shifted left Cnt bits
  Shr           returns Val shifted right Cnt bits
  IAnd          retunrs Int1 AND Int2
  IOr           returns Int1 OR Int2
  IXor          returns Int2 XOR Int2
  Low           returns lower byte of Int
  High          returns upper byte of Int
  Swap          returns Int with upper and lower bytes swapped

  Create        sets Ptr [any pointer] to Addr [any address]
  Address       returns address of Item [any variable]

  MoveDisk      moves (512*Blocks + Bytes) bytes from block BNUm on
                  disk #UNum to Address if Fetch = True;
                  if Fetch = False, moves from memory to disk
  MoveRAM       moves Len bytes from Addr1 to Addr2

  FreeStack     returns free space [in words] from Addr to bottom of
                  program stack
  FreeHeap      returns free space [in words] from top of heap to Addr
  NOTE:         MemAvail = FreeStack(Addr) + FreeHeap(Addr)
                                                                          }
{=========================================================================}

function  Shl(Val,Cnt : Integer) : Integer;
function  Shr(Val,Cnt : Integer) : Integer;
function  IAnd(Int1,Int2 : Integer) : Integer;
function  IOr(Int1,Int2 : Integer) : Integer;
function  IXor(Int1,Int2 : Integer) : Integer;
function  Low(Int : Integer) : Integer;
function  High(Int : Integer) : Integer;
function  Swap(Int : Integer) : Integer;

procedure Create(Addr : Integer; var Ptr);
function  Address(var Item);

procedure MoveDisk(UNum,BNum,Address,Blocks,Bytes : Integer; Fetch : Boolean);
procedure MoveRAM(Addr1,Addr2,Len : Integer);

function  FreeStack(Addr : Integer) : Integer;
function  FreeHeap(Addr : Integer) : Integer;

implementation

{        the following routines are found in LOWLIB.ASM                    }

function  Shl;        external;
function  Shr;        external;
function  IAnd;       external;
function  IOr;        external;
function  IXor;       external;
function  Low;        external;
function  High;       external;
function  Swap;       external;
procedure Create;     external;
function  Address;    external;

{        and the rest of the library routines are right here             }

procedure MoveDisk;
{
        purpose      move data between RAM and disk
        last update  13 Aug 84
}
var
  THeap              : HeapPtr;
  Len                : Integer;
begin
  with THeap do begin
    Addr := Address;
    Len  := 512*Blocks + Bytes;
    if Fetch
      then UnitRead(UNum,Buf^,Len,BNum)
      else UnitWrite(UNum,Buf^,Len,BNum)
  end
end; { of proc MoveDisk }

procedure MoveRAM;
{
        purpose      shift Len bytes from Addr1 to Addr2
        last update  07 Aug 84
}
var
  Loc1,Loc2          : HeapPtr;
begin
  Loc1.Addr := Addr1;
  Loc2.Addr := Addr2;
  if Addr2 > Addr1
    then MoveRight(Loc1.Buf^,Loc2.Buf^,Len)
    else MoveLeft(Loc1.Buf^,Loc2.Buf^,Len)
end; { of proc MoveRAM }

function FreeStack;
{
        purpose      returns free space (in words) above Addr
        last update  07 Aug 84
}
var
  THeap,NHeap        : HeapPtr;
begin
  Mark(THeap.Ptr);                     { save current heap pointer  }
  NHeap.Addr := Addr;
  Release(NHeap.Ptr);                  { set heap pointer to addr   }
  FreeStack = MemAvail;                { and get free space         }
  Release(THeap.Ptr)                   { restore heap pointer       }
end; { of proc FreeStack }

function FreeHeap;
{
        purpose      returns free space (in words) from top of heap to Addr
        last update  07 Aug 84
}
var
  THeap              : HeapPtr;
  Temp               : Integer;
begin
  Mark(THeap.Ptr);                     { get current heap pointer  }
  Temp := Addr - THeap.Addr            { get space in bytes        }
  if Temp < 0                          { adjust for value > 32767  }
    then FreeHeap := 16384 + Shr(1+Temp+32767,1)
    else FreeHeap := Shr(Temp+1)
end; { of proc FreeHeap }

{                   main "body" of unit LowLib                            }
begin
end. { of unit LowLib }

